@@ -129,7 +129,12 @@ load_plugin_info (PeasEngine  *engine,
   if (peas_engine_get_plugin_info (engine, module_name) != NULL)
     _peas_plugin_info_unref (info);
   else
-    engine->priv->plugin_list = g_list_prepend (engine->priv->plugin_list, info);
+    {
+      engine->priv->plugin_list = g_list_prepend (engine->priv->plugin_list,
+                                                  info);
+
+      g_object_notify (G_OBJECT (engine), "plugin-list");
+    }
 }
 
 static void
@@ -157,11 +162,15 @@ load_dir_real (PeasEngine  *engine,
     {
       gchar *filename = g_build_filename (module_dir, dirent, NULL);
 
-      if (g_str_has_suffix (dirent, ".plugin"))
-        load_plugin_info (engine, filename, module_dir, data_dir);
-
-      else if (recursions > 0 && g_file_test (filename, G_FILE_TEST_IS_DIR))
-        load_dir_real (engine, filename, data_dir, recursions - 1);
+      if (g_file_test (filename, G_FILE_TEST_IS_DIR))
+        {
+          if (recursions > 0)
+            load_dir_real (engine, filename, data_dir, recursions - 1);
+        }
+      else if (g_str_has_suffix (dirent, ".plugin"))
+        {
+          load_plugin_info (engine, filename, module_dir, data_dir);
+        }
 
       g_free (filename);
     }
@@ -192,6 +201,8 @@ peas_engine_rescan_plugins (PeasEngine *engine)
       return;
     }
 
+  g_object_freeze_notify (G_OBJECT (engine));
+
   /* Go and read everything from the provided search paths */
   for (item = engine->priv->search_paths; item != NULL; item = item->next)
     {
@@ -199,7 +210,7 @@ peas_engine_rescan_plugins (PeasEngine *engine)
       load_dir_real (engine, sp->module_dir, sp->data_dir, 1);
     }
 
-  g_object_notify (G_OBJECT (engine), "plugin-list");
+  g_object_thaw_notify (G_OBJECT (engine));
 }
 
 /**
@@ -243,8 +254,9 @@ peas_engine_add_search_path (PeasEngine *engine,
    * the plugin list. */
   engine->priv->search_paths = g_list_append (engine->priv->search_paths, sp);
 
+  g_object_freeze_notify (G_OBJECT (engine));
   load_dir_real (engine, sp->module_dir, sp->data_dir, 1);
-  g_object_notify (G_OBJECT (engine), "plugin-list");
+  g_object_thaw_notify (G_OBJECT (engine));
 }
 
 static guint
@@ -295,7 +307,7 @@ static void
 loader_garbage_collect (const gchar *id,
                         LoaderInfo  *info)
 {
-  if (info->loader)
+  if (info != NULL && info->loader != NULL)
     peas_plugin_loader_garbage_collect (info->loader);
 }
 
@@ -632,7 +644,7 @@ get_plugin_loader (PeasEngine     *engine,
 
   if (loader_info->module == NULL)
     {
-      g_hash_table_insert (loaders, loader_id, NULL);
+      g_hash_table_insert (loaders, g_strdup (info->loader), NULL);
       return NULL;
     }
 
@@ -653,7 +665,7 @@ get_plugin_loader (PeasEngine     *engine,
         g_object_unref (loader_info->loader);
 
       g_object_unref (loader_info->module);
-      g_hash_table_insert (loaders, info->loader, NULL);
+      g_hash_table_insert (loaders, g_strdup (info->loader), NULL);
       return NULL;
     }
 
@@ -765,7 +777,8 @@ load_plugin (PeasEngine     *engine,
       dep_info = peas_engine_get_plugin_info (engine, dependencies[i]);
       if (!dep_info)
         {
-          g_warning ("Plugin not found: %s", dependencies[i]);
+          g_warning ("Could not find plugin '%s' for plugin '%s'",
+                     dependencies[i], info->name);
           g_set_error (&info->error,
                        PEAS_PLUGIN_INFO_ERROR,
                        PEAS_PLUGIN_INFO_ERROR_DEP_NOT_FOUND,
@@ -809,6 +822,8 @@ load_plugin (PeasEngine     *engine,
       goto error;
     }
 
+  g_debug ("Loaded plugin '%s'", info->module_name);
+
   return TRUE;
 
 error:
@@ -891,6 +906,8 @@ peas_engine_unload_plugin_real (PeasEngine     *engine,
   peas_plugin_loader_garbage_collect (loader);
   peas_plugin_loader_unload (loader, info);
 
+  g_debug ("Unloaded plugin '%s'", info->module_name);
+
   if (!engine->priv->in_dispose)
     g_object_notify (G_OBJECT (engine), "loaded-plugins");
 }
@@ -930,6 +947,7 @@ peas_engine_provides_extension (PeasEngine     *engine,
 
   g_return_val_if_fail (PEAS_IS_ENGINE (engine), FALSE);
   g_return_val_if_fail (info != NULL, FALSE);
+  g_return_val_if_fail (G_TYPE_IS_INTERFACE (extension_type), FALSE);
 
   if (!peas_plugin_info_is_loaded (info))
     return FALSE;
@@ -966,14 +984,26 @@ peas_engine_create_extensionv (PeasEngine     *engine,
                                GParameter     *parameters)
 {
   PeasPluginLoader *loader;
+  PeasExtension *extension;
 
   g_return_val_if_fail (PEAS_IS_ENGINE (engine), NULL);
   g_return_val_if_fail (info != NULL, NULL);
   g_return_val_if_fail (peas_plugin_info_is_loaded (info), NULL);
+  g_return_val_if_fail (G_TYPE_IS_INTERFACE (extension_type), FALSE);
 
   loader = get_plugin_loader (engine, info);
-  return peas_plugin_loader_create_extension (loader, info, extension_type,
-                                              n_parameters, parameters);
+  extension = peas_plugin_loader_create_extension (loader, info, extension_type,
+                                                   n_parameters, parameters);
+
+  if (!PEAS_IS_EXTENSION (extension))
+    {
+      g_warning ("Plugin '%s' does not provide a '%s' extension",
+                 peas_plugin_info_get_module_name (info),
+                 g_type_name (extension_type));
+      return NULL;
+    }
+
+  return extension;
 }
 
 /**
@@ -1006,12 +1036,19 @@ peas_engine_create_extension_valist (PeasEngine     *engine,
   GParameter *parameters;
   PeasExtension *exten;
 
+  g_return_val_if_fail (PEAS_IS_ENGINE (engine), NULL);
+  g_return_val_if_fail (info != NULL, NULL);
+  g_return_val_if_fail (peas_plugin_info_is_loaded (info), NULL);
+  g_return_val_if_fail (G_TYPE_IS_INTERFACE (extension_type), FALSE);
+
   type_struct = _g_type_struct_ref (extension_type);
 
   if (!_valist_to_parameter_list (extension_type, type_struct, first_property,
                                   var_args, &parameters, &n_parameters))
     {
-      g_return_val_if_reached (NULL);
+      /* Already warned */
+      _g_type_struct_unref (extension_type, type_struct);
+      return NULL;
     }
 
   exten = peas_engine_create_extensionv (engine, info, extension_type,
@@ -1061,6 +1098,11 @@ peas_engine_create_extension (PeasEngine     *engine,
   va_list var_args;
   PeasExtension *exten;
 
+  g_return_val_if_fail (PEAS_IS_ENGINE (engine), NULL);
+  g_return_val_if_fail (info != NULL, NULL);
+  g_return_val_if_fail (peas_plugin_info_is_loaded (info), NULL);
+  g_return_val_if_fail (G_TYPE_IS_INTERFACE (extension_type), FALSE);
+
   va_start (var_args, first_property);
   exten = peas_engine_create_extension_valist (engine, info, extension_type,
                                                first_property, var_args);
