@@ -53,19 +53,17 @@
  * static void
  * on_extension_added (PeasExtensionSet *set,
  *                     PeasPluginInfo   *info,
- *                     PeasExtension    *exten,
- *                     GtkWindow        *window)
+ *                     PeasActivatable  *activatable)
  * {
- *   peas_extension_call (exten, "activate", window);
+ *   peas_activatable_activate (activatable);
  * }
  *
  * static void
  * on_extension_removed (PeasExtensionSet *set,
  *                       PeasPluginInfo   *info,
- *                       PeasExtension    *exten,
- *                       GtkWindow        *window)
+ *                       PeasActivatable  *activatable)
  * {
- *   peas_extension_call (exten, "deactivate", window);
+ *   peas_activatable_deactivate (activatable);
  * }
  *
  * PeasExtensionSet *
@@ -78,9 +76,9 @@
  *                                 "object", window, NULL);
  *   peas_extension_set_call (set, "activate");
  *   g_signal_connect (set, "extension-added",
- *                     G_CALLBACK (on_extension_added), window);
+ *                     G_CALLBACK (on_extension_added), NULL);
  *   g_signal_connect (set, "extension-removed",
- *                     G_CALLBACK (on_extension_removed), window);
+ *                     G_CALLBACK (on_extension_removed), NULL);
  *   return set;
  * }
  * ]|
@@ -200,12 +198,14 @@ add_extension (PeasExtensionSet *set,
   if (!peas_plugin_info_is_loaded (info))
     return;
 
+  if (!peas_engine_provides_extension (set->priv->engine, info,
+                                       set->priv->exten_type))
+    return;
+
   exten = peas_engine_create_extensionv (set->priv->engine, info,
                                          set->priv->exten_type,
                                          set->priv->n_parameters,
                                          set->priv->parameters);
-  if (!exten)
-    return;
 
 /*  peas_plugin_info_ref (info); */
 
@@ -383,7 +383,7 @@ peas_extension_set_class_init (PeasExtensionSetClass *klass)
    * @info: A #PeasPluginInfo.
    * @exten: A #PeasExtension.
    *
-   * The extension-added signal is emitted when a new extension is about to be
+   * The extension-removed signal is emitted when a new extension is about to be
    * removed from the #PeasExtensionSet. It happens when a plugin implementing
    * the extension set's extension type is unloaded, or when the
    * #PeasExtensionSet itself is destroyed.
@@ -519,7 +519,10 @@ peas_extension_set_call_valist (PeasExtensionSet *set,
   g_return_val_if_fail (method_name != NULL, FALSE);
 
   callable_info = peas_gi_get_method_info (set->priv->exten_type, method_name);
-  g_return_val_if_fail (callable_info != NULL, FALSE);
+
+  /* Already warned */
+  if (callable_info == NULL)
+    return FALSE;
 
   n_args = g_callable_info_get_n_args (callable_info);
   g_return_val_if_fail (n_args >= 0, FALSE);
@@ -630,8 +633,9 @@ peas_extension_set_new_valist (PeasEngine  *engine,
   if (!_valist_to_parameter_list (exten_type, type_struct, first_property,
                                   var_args, &parameters, &n_parameters))
     {
-      /* WARNING */
-      g_return_val_if_reached (NULL);
+      /* Already warned */
+      _g_type_struct_unref (exten_type, type_struct);
+      return NULL;
     }
 
   set = peas_extension_set_newv (engine, exten_type, n_parameters, parameters);
