@@ -27,13 +27,24 @@
 
 G_BEGIN_DECLS
 
-GICallableInfo  *peas_method_get_info             (GType        iface_type,
+GICallableInfo  *peas_gi_get_method_info          (GType        iface_type,
                                                    const gchar *method_name);
 
-gboolean        peas_method_apply_valist          (GObject     *instance,
+void             peas_gi_valist_to_arguments      (GICallableInfo *callable_info,
+                                                   va_list         va_args,
+                                                   GIArgument     *arguments,
+                                                   gpointer       *return_value);
+void             peas_gi_argument_to_pointer      (GITypeInfo     *type_info,
+                                                   GIArgument     *arg,
+                                                   gpointer        ptr);
+void             peas_gi_pointer_to_argument      (GITypeInfo     *type_info,
+                                                   gpointer        ptr,
+                                                   GIArgument     *arg);
+gboolean         peas_method_apply                (GObject     *instance,
                                                    GType        iface_type,
                                                    const gchar *method_name,
-                                                   va_list      args);
+                                                   GIArgument  *args,
+                                                   GIArgument  *return_value);
 
 G_END_DECLS
 
