@@ -41,10 +41,26 @@ typedef struct _PeasObjectModule         PeasObjectModule;
 typedef struct _PeasObjectModuleClass    PeasObjectModuleClass;
 typedef struct _PeasObjectModulePrivate  PeasObjectModulePrivate;
 
+/**
+ * PeasFactoryFunc:
+ *
+ * A #PeasFactoryFunc is a factory function which will instanciate a new
+ * extension of a given type. g_object_newv() is such a function.
+ *
+ * It is used with peas_object_module_register_object_factory().
+ *
+ * Return value: (transfer full):
+ */
 typedef GObject *(*PeasFactoryFunc)   (guint          n_parameters,
                                        GParameter    *parameters,
                                        gpointer       user_data);
 
+/**
+ * PeasObjectModule:
+ *
+ * The #PeasObjectModule structure contains only private data and should only
+ * be accessed using the provided API.
+ */
 struct _PeasObjectModule {
   GTypeModule parent;
 
@@ -53,6 +69,8 @@ struct _PeasObjectModule {
 
 struct _PeasObjectModuleClass {
   GTypeModuleClass parent_class;
+
+  gpointer padding[8];
 };
 
 GType               peas_object_module_get_type               (void) G_GNUC_CONST;
