@@ -44,10 +44,11 @@
  *
  * A #PeasPluginInfo contains all the information available about a plugin.
  *
- * All this information comes from the related plugin info file. Here is an
- * example of such a plugin file, in the #GKeyFile format:
+ * All this information comes from the related plugin info file, whose file
+ * extension is ".plugin". Here is an example of such a plugin file, in the
+ * #GKeyFile format:
  * |[
- * [YourApplication Plugin]
+ * [Plugin]
  * Module=helloworld
  * Name=Hello World
  * Description=Displays "Hello World"
@@ -72,9 +73,6 @@ _peas_plugin_info_unref (PeasPluginInfo *info)
   if (!g_atomic_int_dec_and_test (&info->refcount))
     return;
 
-  if (info->keys != NULL)
-    g_hash_table_destroy (info->keys);
-  g_free (info->file);
   g_free (info->module_dir);
   g_free (info->data_dir);
   g_free (info->module_name);
@@ -88,6 +86,8 @@ _peas_plugin_info_unref (PeasPluginInfo *info)
   g_free (info->version);
   g_free (info->help_uri);
   g_strfreev (info->authors);
+  if (info->error != NULL)
+    g_error_free (info->error);
 
   g_free (info);
 }
@@ -105,76 +105,16 @@ peas_plugin_info_get_type (void)
   return the_type;
 }
 
-static void
-value_free (GValue *value)
+GQuark
+peas_plugin_info_error_quark (void)
 {
-  g_value_unset (value);
-  g_free (value);
-}
+  static volatile gsize quark = 0;
 
-static void
-parse_extra_keys (PeasPluginInfo   *info,
-                  GKeyFile         *plugin_file,
-                  const gchar     **keys)
-{
-  guint i;
+	if (g_once_init_enter (&quark))
+		g_once_init_leave (&quark,
+		                   g_quark_from_static_string ("peas-plugin-info-error"));
 
-  for (i = 0; keys[i] != NULL; i++)
-    {
-      GValue *value = NULL;
-      gboolean b;
-      GError *error = NULL;
-
-      if (g_str_equal (keys[i], "IAge") ||
-          g_str_equal (keys[i], "Module") ||
-          g_str_equal (keys[i], "Depends") ||
-          g_str_equal (keys[i], "Loader") ||
-          g_str_equal (keys[i], "Name") ||
-          g_str_has_prefix (keys[i], "Name[") ||
-          g_str_equal (keys[i], "Description") ||
-          g_str_has_prefix (keys[i], "Description[") ||
-          g_str_equal (keys[i], "Icon") ||
-          g_str_equal (keys[i], "Authors") ||
-          g_str_equal (keys[i], "Copyright") ||
-          g_str_equal (keys[i], "Website") ||
-          g_str_equal (keys[i], "Version") ||
-          g_str_has_prefix (keys[i], "Help") ||
-          g_str_equal (keys[i], "Builtin"))
-        continue;
-
-      b = g_key_file_get_boolean (plugin_file, "Plugin", keys[i], &error);
-      if (b == FALSE && error != NULL)
-        {
-          gchar *str;
-          g_error_free (error);
-          error = NULL;
-          str = g_key_file_get_string (plugin_file, "Plugin", keys[i], NULL);
-          if (str != NULL)
-            {
-              value = g_new0 (GValue, 1);
-              g_value_init (value, G_TYPE_STRING);
-              g_value_take_string (value, str);
-            }
-        }
-      else
-        {
-          value = g_new0 (GValue, 1);
-          g_value_init (value, G_TYPE_BOOLEAN);
-          g_value_set_boolean (value, b);
-        }
-
-      if (!value)
-        continue;
-
-      if (info->keys == NULL)
-        {
-          info->keys = g_hash_table_new_full (g_str_hash,
-                                              g_str_equal,
-                                              g_free,
-                                              (GDestroyNotify) value_free);
-        }
-      g_hash_table_insert (info->keys, g_strdup (keys[i]), value);
-    }
+	return quark;
 }
 
 /*
@@ -195,7 +135,6 @@ _peas_plugin_info_new (const gchar *filename,
   PeasPluginInfo *info;
   GKeyFile *plugin_file = NULL;
   gchar *str;
-  gchar **keys;
   gint integer;
   gboolean b;
   GError *error = NULL;
@@ -204,7 +143,6 @@ _peas_plugin_info_new (const gchar *filename,
 
   info = g_new0 (PeasPluginInfo, 1);
   info->refcount = 1;
-  info->file = g_strdup (filename);
 
   plugin_file = g_key_file_new ();
   if (!g_key_file_load_from_file (plugin_file, filename, G_KEY_FILE_NONE, NULL))
@@ -214,7 +152,10 @@ _peas_plugin_info_new (const gchar *filename,
     }
 
   if (!g_key_file_has_key (plugin_file, "Plugin", "IAge", NULL))
-    goto error;
+    {
+      g_warning ("Could not find 'IAge' in '%s'", filename);
+      goto error;
+    }
 
   integer = g_key_file_get_integer (plugin_file, "Plugin", "IAge", NULL);
   info->iage = integer <= 0 ? 0 : integer;
@@ -232,6 +173,17 @@ _peas_plugin_info_new (const gchar *filename,
       goto error;
     }
 
+  /* Get Name */
+  str = g_key_file_get_locale_string (plugin_file, "Plugin",
+                                      "Name", NULL, NULL);
+  if (str)
+    info->name = str;
+  else
+    {
+      g_warning ("Could not find 'Name' in '%s'", filename);
+      goto error;
+    }
+
   /* Get the dependency list */
   info->dependencies = g_key_file_get_string_list (plugin_file,
                                                    "Plugin",
@@ -252,17 +204,6 @@ _peas_plugin_info_new (const gchar *filename,
       info->loader = g_strdup ("C");
     }
 
-  /* Get Name */
-  str = g_key_file_get_locale_string (plugin_file, "Plugin",
-                                      "Name", NULL, NULL);
-  if (str)
-    info->name = str;
-  else
-    {
-      g_warning ("Could not find 'Name' in '%s'", filename);
-      goto error;
-    }
-
   /* Get Description */
   str = g_key_file_get_locale_string (plugin_file, "Plugin",
                                       "Description", NULL, NULL);
@@ -312,10 +253,12 @@ _peas_plugin_info_new (const gchar *filename,
   else
     info->builtin = b;
 
-  /* Get extra keys */
-  keys = g_key_file_get_keys (plugin_file, "Plugin", NULL, NULL);
-  parse_extra_keys (info, plugin_file, (const gchar **) keys);
-  g_strfreev (keys);
+  /* Get Hidden */
+  b = g_key_file_get_boolean (plugin_file, "Plugin", "Hidden", &error);
+  if (error != NULL)
+    g_clear_error (&error);
+  else
+    info->hidden = b;
 
   g_key_file_free (plugin_file);
 
@@ -329,10 +272,8 @@ _peas_plugin_info_new (const gchar *filename,
   return info;
 
 error:
-  g_free (info->file);
   g_free (info->module_name);
   g_free (info->name);
-  g_free (info->loader);
   g_free (info);
   g_key_file_free (plugin_file);
 
@@ -358,19 +299,28 @@ peas_plugin_info_is_loaded (const PeasPluginInfo *info)
 /**
  * peas_plugin_info_is_available:
  * @info: A #PeasPluginInfo.
+ * @error: A #GError.
  *
  * Check if the plugin is available.
  *
  * A plugin is marked as not available when there is no loader available to
  * load it, or when there has been an error when trying to load it previously.
+ * If not available then @error will be set.
  *
  * Returns: %TRUE if the plugin is available.
  */
 gboolean
-peas_plugin_info_is_available (const PeasPluginInfo *info)
+peas_plugin_info_is_available (const PeasPluginInfo  *info,
+                               GError               **error)
 {
   g_return_val_if_fail (info != NULL, FALSE);
 
+  /* Uses g_propagate_error() so we get the right warning
+   * in the case that *error != NULL
+   */
+  if (error != NULL && info->error != NULL)
+    g_propagate_error (error, g_error_copy (info->error));
+
   return info->available != FALSE;
 }
 
@@ -378,10 +328,10 @@ peas_plugin_info_is_available (const PeasPluginInfo *info)
  * peas_plugin_info_is_builtin:
  * @info: A #PeasPluginInfo.
  *
- * Gets is the plugin is a builtin plugin.
+ * Check if the plugin is a builtin plugin.
  *
  * A builtin plugin is a plugin which cannot be enabled or disabled by the
- * user through a plugin manager (like #PeasUIPluginManager). Loading or
+ * user through a plugin manager (like #PeasGtkPluginManager). Loading or
  * unloading such plugins is the responsibility of the application alone.
  * Most applications will usually load those plugins immediately after
  * the initialization of the #PeasEngine.
@@ -399,6 +349,30 @@ peas_plugin_info_is_builtin (const PeasPluginInfo *info)
   return info->builtin;
 }
 
+/**
+ * peas_plugin_info_is_hidden:
+ * @info: A #PeasPluginInfo.
+ *
+ * Check if the plugin is a hidden plugin.
+ *
+ * A hidden plugin is a plugin which cannot be seen by a
+ * user through a plugin manager (like #PeasGtkPluginManager). Loading and
+ * unloading such plugins is the responsibility of the application alone or
+ * through plugins that depend on them.
+ *
+ * The relevant key in the plugin info file is "Hidden".
+ *
+ * Returns: %TRUE if the plugin is a hidden plugin, %FALSE
+ * if not.
+ **/
+gboolean
+peas_plugin_info_is_hidden (const PeasPluginInfo *info)
+{
+  g_return_val_if_fail (info != NULL, FALSE);
+
+  return info->hidden;
+}
+
 /**
  * peas_plugin_info_get_module_name:
  * @info: A #PeasPluginInfo.
@@ -582,11 +556,11 @@ peas_plugin_info_get_icon_name (const PeasPluginInfo *info)
  * peas_plugin_info_get_authors:
  * @info: A #PeasPluginInfo.
  *
- * Gets a NULL-terminated array of strings with the authors of the plugin.
+ * Gets a %NULL-terminated array of strings with the authors of the plugin.
  *
  * The relevant key in the plugin info file is "Authors".
  *
- * Returns: (transfer none): the plugin's author list.
+ * Returns: (transfer none) (array zero-terminated=1): the plugin's author list.
  */
 const gchar **
 peas_plugin_info_get_authors (const PeasPluginInfo *info)
@@ -661,8 +635,8 @@ peas_plugin_info_get_version (const PeasPluginInfo *info)
  * website or a ghelp: URI if a Gnome help page is available for the plugin.
  *
  * The relevant key in the plugin info file is "Help". Other platform-specific
- * keys exist for platform-specific help files. Those are "Help-Gnome",
- * "Help-Windows" and "Help-MacOs-X".
+ * keys exist for platform-specific help files. Those are "Help-GNOME",
+ * "Help-Windows" and "Help-MacOS-X".
  *
  * Returns: the plugin's help URI.
  */
@@ -691,24 +665,3 @@ peas_plugin_info_get_iage (const PeasPluginInfo *info)
 
   return info->iage;
 }
-
-/**
- * peas_plugin_info_get_keys:
- * @info: A #PeasPluginInfo.
- *
- * Gets a hash table of string keys present and #GValue values,
- * present in the plugin information file, but not handled
- * by libpeas. Note that libpeas only handles booleans and
- * strings, and that strings that are recognized as booleans,
- * as done by g_key_file_get_boolean(), will be of boolean type.
- *
- * Returns: a #GHashTable of string keys and #GValue values. Do
- * not free or destroy any data in this hashtable.
- **/
-const GHashTable *
-peas_plugin_info_get_keys (const PeasPluginInfo *info)
-{
-  g_return_val_if_fail (info != NULL, NULL);
-
-  return info->keys;
-}
