@@ -54,7 +54,7 @@ typedef struct {
 
 static PyObject *PyGObject_Type;
 
-static gboolean   peas_plugin_loader_python_add_module_path (PeasPluginLoaderPython *self,
+static gboolean   peas_plugin_loader_python_add_module_path (PeasPluginLoaderPython *pyloader,
                                                              const gchar            *module_path);
 
 G_DEFINE_TYPE (PeasPluginLoaderPython, peas_plugin_loader_python, PEAS_TYPE_PLUGIN_LOADER);
@@ -81,33 +81,32 @@ find_python_extension_type (PeasPluginInfo *info,
 
   pygtype = pyg_type_wrapper_new (exten_type);
   pytype = PyObject_GetAttrString (pygtype, "pytype");
-  g_return_val_if_fail (pytype != NULL, NULL);
+  g_warn_if_fail (pytype != NULL);
 
-  if (pytype == Py_None)
-    return NULL;
-
-  while (PyDict_Next (locals, &pos, &key, &value))
+  if (pytype != NULL && pytype != Py_None)
     {
-      if (!PyType_Check (value))
-        continue;
-
-      switch (PyObject_IsSubclass (value, pytype))
+      while (PyDict_Next (locals, &pos, &key, &value))
         {
-        case 1:
-          Py_DECREF (pygtype);
-          return (PyTypeObject *) value;
-        case 0:
-          continue;
-        case -1:
-        default:
-          PyErr_Print ();
-          continue;
+          if (!PyType_Check (value))
+            continue;
+
+          switch (PyObject_IsSubclass (value, pytype))
+            {
+            case 1:
+              Py_DECREF (pygtype);
+              return (PyTypeObject *) value;
+            case 0:
+              continue;
+            case -1:
+            default:
+              PyErr_Print ();
+              continue;
+            }
         }
     }
 
   Py_DECREF (pygtype);
-  g_debug ("No '%s' derivative found in Python plugin '%s'",
-           g_type_name (exten_type), peas_plugin_info_get_name (info));
+
   return NULL;
 }
 
@@ -123,9 +122,6 @@ peas_plugin_loader_python_provides_extension (PeasPluginLoader *loader,
 
   pyinfo = (PythonInfo *) g_hash_table_lookup (pyloader->priv->loaded_plugins, info);
 
-  if (pyinfo == NULL)
-    return FALSE;
-
   state = pyg_gil_state_ensure ();
   extension_type = find_python_extension_type (info, exten_type, pyinfo->module);
   pyg_gil_state_release (state);
@@ -143,75 +139,40 @@ peas_plugin_loader_python_create_extension (PeasPluginLoader *loader,
   PeasPluginLoaderPython *pyloader = PEAS_PLUGIN_LOADER_PYTHON (loader);
   PythonInfo *pyinfo;
   PyTypeObject *pytype;
+  GType the_type;
+  GObject *object;
   PyObject *pyobject;
-  PyGObject *pygobject;
-  PyObject *emptyarg;
   PyObject *pyplinfo;
   PyGILState_STATE state;
-  PeasExtension *exten;
+  PeasExtension *exten = NULL;
 
   pyinfo = (PythonInfo *) g_hash_table_lookup (pyloader->priv->loaded_plugins, info);
 
-  if (pyinfo == NULL)
-    return NULL;
-
   state = pyg_gil_state_ensure ();
 
   pytype = find_python_extension_type (info, exten_type, pyinfo->module);
 
-  if (pytype == NULL || pytype->tp_new == NULL)
+  if (pytype == NULL)
+    goto out;
+
+  the_type = pyg_type_from_object ((PyObject *) pytype);
+
+  if (the_type == G_TYPE_INVALID)
+    goto out;
+
+  if (!g_type_is_a (the_type, exten_type))
     {
-      pyg_gil_state_release (state);
-      return NULL;
+      g_warn_if_fail (g_type_is_a (the_type, exten_type));
+      goto out;
     }
 
-  emptyarg = PyTuple_New (0);
-  pyobject = pytype->tp_new (pytype, emptyarg, NULL);
-  Py_DECREF (emptyarg);
+  object = g_object_newv (the_type, n_parameters, parameters);
 
-  if (pyobject == NULL)
-    {
-      pyg_gil_state_release (state);
-      g_warning ("Could not create instance for '%s'",
-                 peas_plugin_info_get_name (info));
+  if (!object)
+    goto out;
 
-      return NULL;
-    }
-
-  pygobject = (PyGObject *) pyobject;
-
-  if (pygobject->obj != NULL)
-    {
-      Py_DECREF (pyobject);
-      pyg_gil_state_release (state);
-      g_warning ("Could not create instance for '%s': GObject already initialized",
-                 peas_plugin_info_get_name (info));
-
-      return NULL;
-    }
-
-  pygobject_constructv (pygobject, n_parameters, parameters);
-
-  if (pygobject->obj == NULL)
-    {
-      Py_DECREF (pyobject);
-      pyg_gil_state_release (state);
-      g_warning ("Could not create '%s' instance for '%s': GObject not constructed",
-                 g_type_name (exten_type), peas_plugin_info_get_name (info));
-
-      return NULL;
-    }
-
-  g_return_val_if_fail (G_TYPE_CHECK_INSTANCE_TYPE (pygobject->obj, exten_type), NULL);
-
-  /* now call tp_init manually */
-  if (PyType_IsSubtype (pyobject->ob_type, pytype) &&
-      pyobject->ob_type->tp_init != NULL)
-    {
-      emptyarg = PyTuple_New (0);
-      pyobject->ob_type->tp_init (pyobject, emptyarg, NULL);
-      Py_DECREF (emptyarg);
-    }
+  pyobject = pygobject_new (object);
+  g_object_unref (object);
 
   /* Set the plugin info as an attribute of the instance */
   pyplinfo = pyg_boxed_new (PEAS_TYPE_PLUGIN_INFO, info, TRUE, TRUE);
@@ -221,6 +182,8 @@ peas_plugin_loader_python_create_extension (PeasPluginLoader *loader,
   exten = peas_extension_python_new (exten_type, pyobject);
   Py_DECREF (pyobject);
 
+out:
+
   pyg_gil_state_release (state);
 
   return exten;
@@ -241,26 +204,6 @@ add_python_info (PeasPluginLoaderPython *loader,
   g_hash_table_insert (loader->priv->loaded_plugins, info, pyinfo);
 }
 
-static void
-peas_plugin_loader_python_add_module_directory (PeasPluginLoader *loader,
-                                                const gchar      *module_dir)
-{
-  PeasPluginLoaderPython *pyloader = PEAS_PLUGIN_LOADER_PYTHON (loader);
-  PyGILState_STATE state;
-
-  /* Bail if we failed to initialise Python, since adding a module path won't
-   * help us, and calling the GIL functions will cause a crash. */
-  if (pyloader->priv->init_failed)
-    return;
-
-  state = pyg_gil_state_ensure ();
-
-  g_debug ("Adding '%s' as a module path for the Python loader", module_dir);
-  peas_plugin_loader_python_add_module_path (pyloader, module_dir);
-
-  pyg_gil_state_release (state);
-}
-
 static gboolean
 peas_plugin_loader_python_load (PeasPluginLoader *loader,
                                 PeasPluginInfo   *info)
@@ -308,6 +251,8 @@ peas_plugin_loader_python_load (PeasPluginLoader *loader,
 
   add_python_info (pyloader, info, pymodule);
 
+  Py_DECREF (pymodule);
+
   pyg_gil_state_release (state);
 
   return TRUE;
@@ -318,17 +263,6 @@ peas_plugin_loader_python_unload (PeasPluginLoader *loader,
                                   PeasPluginInfo   *info)
 {
   PeasPluginLoaderPython *pyloader = PEAS_PLUGIN_LOADER_PYTHON (loader);
-  PythonInfo *pyinfo;
-  PyGILState_STATE state;
-
-  pyinfo = (PythonInfo *) g_hash_table_lookup (pyloader->priv->loaded_plugins, info);
-
-  if (!pyinfo)
-    return;
-
-  state = pyg_gil_state_ensure ();
-  Py_XDECREF (pyinfo->module);
-  pyg_gil_state_release (state);
 
   g_hash_table_remove (pyloader->priv->loaded_plugins, info);
 }
@@ -356,13 +290,11 @@ run_gc (PeasPluginLoaderPython *loader)
 static void
 peas_plugin_loader_python_garbage_collect (PeasPluginLoader *loader)
 {
-  PeasPluginLoaderPython *pyloader;
+  PeasPluginLoaderPython *pyloader = PEAS_PLUGIN_LOADER_PYTHON (loader);
 
-  if (!Py_IsInitialized ())
+  if (pyloader->priv->init_failed)
     return;
 
-  pyloader = PEAS_PLUGIN_LOADER_PYTHON (loader);
-
   /*
    * We both run the GC right now and we schedule
    * a further collection in the main loop.
@@ -406,19 +338,24 @@ peas_python_shutdown (PeasPluginLoaderPython *loader)
  */
 /* NOTE: This must be called with the GIL held */
 static gboolean
-peas_plugin_loader_python_add_module_path (PeasPluginLoaderPython *self,
+peas_plugin_loader_python_add_module_path (PeasPluginLoaderPython *pyloader,
                                            const gchar            *module_path)
 {
   PyObject *pathlist, *pathstring;
 
-  g_return_val_if_fail (PEAS_IS_PLUGIN_LOADER_PYTHON (self), FALSE);
+  g_return_val_if_fail (PEAS_IS_PLUGIN_LOADER_PYTHON (pyloader), FALSE);
   g_return_val_if_fail (module_path != NULL, FALSE);
 
-  if (!Py_IsInitialized ())
+  if (pyloader->priv->init_failed)
     return FALSE;
 
-  pathlist = PySys_GetObject ((char*) "path");
+  pathlist = PySys_GetObject ((char *) "path");
+
+#if PY_VERSION_HEX < 0x03000000
   pathstring = PyString_FromString (module_path);
+#else
+  pathstring = PyUnicode_FromString (module_path);
+#endif
 
   if (PySequence_Contains (pathlist, pathstring) == 0)
     PyList_Insert (pathlist, 0, pathstring);
@@ -427,12 +364,42 @@ peas_plugin_loader_python_add_module_path (PeasPluginLoaderPython *self,
   return TRUE;
 }
 
+#if PY_VERSION_HEX >= 0x03000000
+static wchar_t *
+peas_wchar_from_str (const gchar *str)
+{
+  wchar_t *outbuf;
+  gsize argsize, count;
+
+  argsize = mbstowcs (NULL, str, 0);
+  if (argsize == (gsize)-1)
+    {
+      g_warning ("Could not convert argument to wchar_t string.");
+      return NULL;
+    }
+
+  outbuf = g_new (wchar_t, argsize + 1);
+  count = mbstowcs (outbuf, str, argsize + 1);
+  if (count == (gsize)-1)
+    {
+      g_warning ("Could not convert argument to wchar_t string.");
+      return NULL;
+    }
+
+  return outbuf;
+}
+#endif
+
 static gboolean
 peas_python_init (PeasPluginLoaderPython *loader)
 {
   PyObject *mdict, *gobject, *gi, *gettext, *install, *gettext_args;
-  const char *argv[] = { "", NULL };
   gchar *prgname;
+#if PY_VERSION_HEX < 0x03000000
+  const char *argv[] = { "", NULL };
+#else
+  wchar_t *argv[] = { L"", NULL };
+#endif
 
   /* We are trying to initialize Python for the first time,
      set init_failed to FALSE only if the entire initialization process
@@ -448,13 +415,27 @@ peas_python_init (PeasPluginLoaderPython *loader)
 
   prgname = g_get_prgname ();
   if (prgname != NULL)
-    argv[0] = prgname;
+    {
+#if PY_VERSION_HEX < 0x03000000
+      argv[0] = prgname;
+#else
+      argv[0] = peas_wchar_from_str (prgname);
+#endif
+    }
 
+  /* See http://docs.python.org/c-api/init.html#PySys_SetArgvEx */
 #if PY_VERSION_HEX < 0x02060600
   PySys_SetArgv (1, (char**) argv);
   PyRun_SimpleString ("import sys; sys.path.pop(0)\n");
-#else
+#elif PY_VERSION_HEX < 0x03000000
   PySys_SetArgvEx (1, (char**) argv, 0);
+#elif PY_VERSION_HEX < 0x03010300
+  PySys_SetArgv (1, argv);
+  PyRun_SimpleString ("import sys; sys.path.pop(0)\n");
+  g_free (argv[0]);
+#else
+  PySys_SetArgvEx (1, argv, 0);
+  g_free (argv[0]);
 #endif
 
   /* Note that we don't call this with the GIL held, since we haven't initialised pygobject yet */
@@ -540,27 +521,29 @@ static void
 destroy_python_info (PythonInfo *info)
 {
   PyGILState_STATE state = pyg_gil_state_ensure ();
-  Py_XDECREF (info->module);
+
+  Py_DECREF (info->module);
+
   pyg_gil_state_release (state);
 
   g_free (info);
 }
 
 static void
-peas_plugin_loader_python_init (PeasPluginLoaderPython *self)
+peas_plugin_loader_python_init (PeasPluginLoaderPython *pyloader)
 {
-  self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self,
-                                            PEAS_TYPE_PLUGIN_LOADER_PYTHON,
-                                            PeasPluginLoaderPythonPrivate);
+  pyloader->priv = G_TYPE_INSTANCE_GET_PRIVATE (pyloader,
+                                                PEAS_TYPE_PLUGIN_LOADER_PYTHON,
+                                                PeasPluginLoaderPythonPrivate);
 
   /* initialize python interpreter */
-  peas_python_init (self);
+  peas_python_init (pyloader);
 
-  /* loaded_plugins maps EggPluginsInfo to a PythonInfo */
-  self->priv->loaded_plugins = g_hash_table_new_full (g_direct_hash,
-                                                      g_direct_equal,
-                                                      NULL,
-                                                      (GDestroyNotify) destroy_python_info);
+  /* loaded_plugins maps PeasPluginInfo to a PythonInfo */
+  pyloader->priv->loaded_plugins = g_hash_table_new_full (g_direct_hash,
+                                                          g_direct_equal,
+                                                          NULL,
+                                                          (GDestroyNotify) destroy_python_info);
 }
 
 static void
@@ -582,7 +565,6 @@ peas_plugin_loader_python_class_init (PeasPluginLoaderPythonClass *klass)
 
   object_class->finalize = peas_plugin_loader_python_finalize;
 
-  loader_class->add_module_directory = peas_plugin_loader_python_add_module_directory;
   loader_class->load = peas_plugin_loader_python_load;
   loader_class->unload = peas_plugin_loader_python_unload;
   loader_class->create_extension = peas_plugin_loader_python_create_extension;
