@@ -32,7 +32,7 @@ SeedValue seed_value_from_gvalue (SeedContext    ctx,
 #include "peas-plugin-loader-seed.h"
 #include "peas-extension-seed.h"
 
-G_DEFINE_DYNAMIC_TYPE (PeasPluginLoaderSeed, peas_plugin_loader_seed, PEAS_TYPE_PLUGIN_LOADER);
+G_DEFINE_TYPE (PeasPluginLoaderSeed, peas_plugin_loader_seed, PEAS_TYPE_PLUGIN_LOADER);
 
 typedef struct {
   SeedContext context;
@@ -41,28 +41,6 @@ typedef struct {
 
 static SeedEngine *seed = NULL;
 
-static void
-peas_plugin_loader_seed_add_module_directory (PeasPluginLoader *loader,
-                                              const gchar      *module_dir)
-{
-  gchar **sp = seed_engine_get_search_path (seed);
-
-  if (sp)
-    {
-      gchar *orig_sp = g_strjoinv (":", sp);
-      gchar *new_sp = g_strconcat (module_dir, ":", orig_sp, NULL);
-
-      seed_engine_set_search_path (seed, new_sp);
-
-      g_free (new_sp);
-      g_free (orig_sp);
-    }
-  else
-    {
-      seed_engine_set_search_path (seed, module_dir);
-    }
-}
-
 static gchar *
 get_script_for_plugin_info (PeasPluginInfo   *info,
                             SeedContext       context)
@@ -140,16 +118,14 @@ peas_plugin_loader_seed_provides_extension  (PeasPluginLoader *loader,
 {
   PeasPluginLoaderSeed *sloader = PEAS_PLUGIN_LOADER_SEED (loader);
   SeedInfo *sinfo;
-  SeedObject *extension;
+  SeedValue extension;
 
   sinfo = (SeedInfo *) g_hash_table_lookup (sloader->loaded_plugins, info);
-  if (!sinfo)
-    return FALSE;
 
   extension = seed_object_get_property (sinfo->context,
                                         sinfo->extensions,
                                         g_type_name (exten_type));
-  return extension != NULL;
+  return extension && seed_value_is_object (sinfo->context, extension);
 }
 
 static PeasExtension *
@@ -166,14 +142,14 @@ peas_plugin_loader_seed_create_extension (PeasPluginLoader *loader,
   guint i, j;
 
   sinfo = (SeedInfo *) g_hash_table_lookup (sloader->loaded_plugins, info);
-  if (!sinfo)
-    return NULL;
 
   /* FIXME: instantiate new object and pass the parameters */
   extension_methods = seed_object_get_property (sinfo->context,
                                                 sinfo->extensions,
                                                 g_type_name (exten_type));
-  if (!extension_methods)
+  if (!extension_methods ||
+      seed_value_is_undefined (sinfo->context, extension_methods) ||
+      seed_value_is_null (sinfo->context, extension_methods))
     return NULL;
 
   if (!seed_value_is_object (sinfo->context, extension_methods))
@@ -234,16 +210,8 @@ peas_plugin_loader_seed_unload (PeasPluginLoader *loader,
                                 PeasPluginInfo   *info)
 {
   PeasPluginLoaderSeed *sloader = PEAS_PLUGIN_LOADER_SEED (loader);
-  SeedInfo *sinfo;
-
-  sinfo = (SeedInfo *) g_hash_table_lookup (sloader->loaded_plugins, info);
-  if (!sinfo)
-    return;
 
   g_hash_table_remove (sloader->loaded_plugins, info);
-  seed_value_unprotect (sinfo->context, sinfo->extensions);
-  seed_context_unref (sinfo->context);
-  g_slice_free (SeedInfo, sinfo);
 }
 
 static void
@@ -252,6 +220,15 @@ peas_plugin_loader_seed_garbage_collect (PeasPluginLoader *loader)
   seed_context_collect (seed->context);
 }
 
+static void
+destroy_seed_info (SeedInfo *info)
+{
+  seed_value_unprotect (info->context, info->extensions);
+  seed_context_unref (info->context);
+
+  g_slice_free (SeedInfo, info);
+}
+
 static void
 peas_plugin_loader_seed_init (PeasPluginLoaderSeed *sloader)
 {
@@ -261,15 +238,29 @@ peas_plugin_loader_seed_init (PeasPluginLoaderSeed *sloader)
   if (!seed)
     seed = seed_init (NULL, NULL);
 
-  sloader->loaded_plugins = g_hash_table_new (g_direct_hash, g_direct_equal);
+  sloader->loaded_plugins = g_hash_table_new_full (g_direct_hash, g_direct_equal,
+                                                   NULL,
+                                                   (GDestroyNotify) destroy_seed_info);
+}
+
+static void
+peas_plugin_loader_seed_finalize (GObject *object)
+{
+  PeasPluginLoaderSeed *sloader = PEAS_PLUGIN_LOADER_SEED (object);
+
+  g_hash_table_destroy (sloader->loaded_plugins);
+
+  G_OBJECT_CLASS (peas_plugin_loader_seed_parent_class)->finalize (object);
 }
 
 static void
 peas_plugin_loader_seed_class_init (PeasPluginLoaderSeedClass *klass)
 {
+  GObjectClass *object_class = G_OBJECT_CLASS (klass);
   PeasPluginLoaderClass *loader_class = PEAS_PLUGIN_LOADER_CLASS (klass);
 
-  loader_class->add_module_directory = peas_plugin_loader_seed_add_module_directory;
+  object_class->finalize = peas_plugin_loader_seed_finalize;
+
   loader_class->load = peas_plugin_loader_seed_load;
   loader_class->provides_extension = peas_plugin_loader_seed_provides_extension;
   loader_class->create_extension = peas_plugin_loader_seed_create_extension;
@@ -277,17 +268,9 @@ peas_plugin_loader_seed_class_init (PeasPluginLoaderSeedClass *klass)
   loader_class->garbage_collect = peas_plugin_loader_seed_garbage_collect;
 }
 
-static void
-peas_plugin_loader_seed_class_finalize (PeasPluginLoaderSeedClass *klass)
-{
-}
-
 G_MODULE_EXPORT void
 peas_register_types (PeasObjectModule *module)
 {
-  peas_plugin_loader_seed_register_type (G_TYPE_MODULE (module));
-  peas_extension_seed_register (G_TYPE_MODULE (module));
-
   peas_object_module_register_extension_type (module,
                                               PEAS_TYPE_PLUGIN_LOADER,
                                               PEAS_TYPE_PLUGIN_LOADER_SEED);
