@@ -35,6 +35,7 @@ typedef struct _TestFixture TestFixture;
 struct _TestFixture {
   PeasEngine *engine;
   GtkWidget *window;
+  GtkWindowGroup *window_group;
   PeasGtkPluginManager *manager;
   PeasGtkPluginManagerView *view;
   GtkTreeSelection *selection;
@@ -69,6 +70,7 @@ test_setup (TestFixture   *fixture,
 
   fixture->engine = testing_engine_new ();
   fixture->window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
+  fixture->window_group = gtk_window_group_new ();
   fixture->manager = PEAS_GTK_PLUGIN_MANAGER (peas_gtk_plugin_manager_new (NULL));
   fixture->view = PEAS_GTK_PLUGIN_MANAGER_VIEW (peas_gtk_plugin_manager_get_view (fixture->manager));
   fixture->selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (fixture->view));
@@ -77,7 +79,7 @@ test_setup (TestFixture   *fixture,
    * window group unless one has already been created and
    * find_window_by_title() will need need it to be in the group
    */
-  gtk_window_group_add_window (gtk_window_group_new (),
+  gtk_window_group_add_window (fixture->window_group,
                                GTK_WINDOW (fixture->window));
 
   gtk_container_add (GTK_CONTAINER (fixture->window),
@@ -123,7 +125,8 @@ static void
 test_teardown (TestFixture   *fixture,
                gconstpointer  data)
 {
-  gtk_widget_destroy (GTK_WIDGET (fixture->manager));
+  gtk_widget_destroy (GTK_WIDGET (fixture->window));
+  g_object_unref (fixture->window_group);
 
   testing_engine_free (fixture->engine);
 }
@@ -169,6 +172,8 @@ test_gtk_plugin_manager_about_button_sensitivity (TestFixture *fixture)
   GtkTreeIter iter;
   PeasPluginInfo *info;
 
+  testing_util_push_log_hook ("Could not find plugin 'does-not-exist' for plugin '*'");
+
   /* Must come first otherwise the first iter may
    * be after a revealed builtin plugin
    */
@@ -195,6 +200,8 @@ test_gtk_plugin_manager_configure_button_sensitivity (TestFixture *fixture)
   GtkTreeIter iter;
   PeasPluginInfo *info;
 
+  testing_util_push_log_hook ("Could not find plugin 'does-not-exist' for plugin '*'");
+
   /* Must come first otherwise the first iter may
    * be after a revealed builtin plugin
    */
@@ -312,6 +319,8 @@ test_gtk_plugin_manager_about_dialog (TestFixture *fixture)
 
   for (i = 0; authors_plugin[i] == NULL && authors_dialog[i] == NULL; ++i)
     g_assert_cmpstr (authors_plugin[i], ==, authors_dialog[i]);
+
+  gtk_widget_destroy (window);
 }
 
 static void
@@ -380,6 +389,8 @@ test_gtk_plugin_manager_configure_dialog (TestFixture *fixture)
   g_assert (help_button != NULL);
 
   g_list_free (list);
+
+  gtk_widget_destroy (window);
 }
 
 static void
@@ -444,5 +455,5 @@ main (int    argc,
 
 #undef TEST
 
-  return g_test_run ();
+  return testing_run_tests ();
 }
