@@ -35,11 +35,12 @@ main (int   argc,
 
   g_type_init ();
 
-  testing_init ();
+  _EXTENSION_TESTS_INIT ("seed");
 
-  peas_engine_enable_loader (peas_engine_get_default (), "seed");
+  _EXTENSION_TEST ("seed", "garbage-collect", garbage_collect);
 
-  testing_extension_set_plugin_ ("extension-" "seed");
+  _EXTENSION_TEST ("seed", "provides-valid", provides_valid);
+  _EXTENSION_TEST ("seed", "provides-invalid", provides_invalid);
 
   _EXTENSION_TEST ("seed", "create-valid", create_valid);
   _EXTENSION_TEST ("seed", "create-invalid", create_invalid);
@@ -63,5 +64,5 @@ main (int   argc,
   _EXTENSION_TEST ("seed", "properties-readwrite", properties_readwrite);
 #endif
 
-  return g_test_run ();
+  return testing_run_tests ();
 }
